;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xabb947dbee78a7450bbc9e65a97141bf . "./code.cg2js/cg-common-control.fasl")
(#xdb1e7d0e01a92a257a8b0b5065f7390a . "./code.cg2js/cg-clipboard-stack.fasl")
(#x46119efa8bd98a331e5790f01dbbdb38 . "./code.cg2js/cg-yes-no-string.fasl")
(#x0b1f9452c429ed25d24537c2740b7d23 . "./code.cg2js/cg-clipboard.fasl")
(#xefae1cde640f8c57b70fd22d5b27abbb . "./code.cg2js/cg-dde-utils.fasl")
(#xb63be116673677ac8e88d33cf7dae1b0 . "./code.cg2js/cg-editable-text.fasl")
(#xbf3b330f895ae1b28bfae150b735f052 . "./code.cg2js/cg-player.fasl")
(#xf77f5db7a6f86e4c3da0d912c5579cde . "./code.cg2js/cg-text-or-combo.fasl")
(#x392275a66922532eb1590fbf962a640b . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x6c2919d387e4029b4462021a4b3c239f . "./code.cg2js/cg-graphics-context.fasl")
(#xbecb73382550cc149d480b3b557ad23c . "./code.cg2js/cg-choose-printer.fasl")
(#x13bb6fd9c856c67219c85757ae5439bc . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xb9c30d63fdd9fa5c2159a90d4c916f39 . "./code.cg2js/cg-edit-in-place.fasl")
(#x5c66837158e4f35f601ba5a66ba2067a . "./code.cg2js/cg-split-bar.fasl")
(#x8e0a2eaa9c91a15f518c185458df2aaa . "./code.cg2js/cg-timer.fasl")
(#x46a5bc38546ed0acea737bbcfdaa8797 . "./code.cg2js/cg-progress-indicator.fasl")
(#x0c833a5fa2249e5e66296edc8687ded7 . "./code.cg2js/cg-object-editor.fasl")
(#xce6781b0ad1062dad64091f0b4025279 . "./code.cg2js/cg-fill-texture.fasl")
(#xb0a2c6b229572ee9f76a760f16ad8983 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x82984e28ef5f3ce5d92ab90c3cd5633e . "./code.cg2js/cg-os-window.fasl")
(#xa730a86caaba74b4ca13aee5fc4a923d . "./code.cg2js/cg-color-dialog.fasl")
(#xcbe5c324c8eaf74571f416d03174142d . "./code.cg2js/cg-combo-box.fasl")
(#xb347abdea8e6109ce4e1ba37727715d4 . "./code.cg2js/cg-directory-dialog.fasl")
(#x565f0c30951d8192a2ecd045e139c6a0 . "./code.cg2js/cg-tooltip.fasl")
(#x18deb207afae7bc2cbe3c57e11582fe5 . "./code.cg2js/cg-lettered-menu.fasl")
(#x0bc1009e1320ba6083e45811eb491919 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x1037dae572bbad33391ecca38761b47e . "./code.cg2js/cg-pixmap.fasl")
(#x7755d37631df6fa8b9c99d3aa0b95b16 . "./code.cg2js/cg.fasl")
(#x5334faddf4b5137b05829b0fbd428b5d . "./code.cg2js/cg-class-slot-grid.fasl")
(#x3920d57876e654a316dd5fbcfd904aaf . "./code.cg2js/cg-template-string.fasl")
(#x76f6fa249bbe4e4280a6a0c2efd73493 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x6931d6e61f2b26aabc299f19b0382229 . "./code.cg2js/cg-check-box.fasl")
(#x6b3ed541d6650bfc5214bdbcc34531ae . "./code.cg2js/cg-toggling-widget.fasl")
(#x3abf36a31ace1337c5ff68e7279e9b1b . "./code.cg2js/cg-radio-button.fasl")
(#xe00ab5ffa575c91b775e0b491eab8869 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x7f01cae62d2c5a9b9ee33d1441a23f25 . "./code.cg2js/cg-scaling-stream.fasl")
(#xc5341f5ac5bd0b73d5ad513140168eab . "./code.cg2js/cg-multi-picture-button.fasl")
(#x4240b83a24305f65b194e0b6029e3c8c . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xfd25eb8a3a139e82a31617a4dd183ef2 . "./code.cg2js/cg-grid-widget.fasl")
(#x23fc10adcae6233862ec7ae6afbfd185 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x15fbd1255e9b3a2c6025d86d84780310 . "./code.cg2js/cg-item-list.fasl")
(#x079bd023ea025f686512091623b92d2b . "./code.cg2js/cg-paren-matching.fasl")
(#x13a91a492b86c6ba7d23e771e923a614 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x9c956fa0d7730cd2645e50980737fa7f . "./code.cg2js/cg-html-dialog.fasl")
(#xa678c9e505e7a4bec27849a87c483f88 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x80423a46ce070235ebfdfc48c8b65b7f . "./code.cg2js/cg-project-window.fasl")
(#x89cfcba9b2705e980e5ea729af87d634 . "./code.cg2js/cg-toolbar.fasl")
(#xb838769a4bf13bbc2358ff405aed49c3 . "./code.cg2js/cg-caret.fasl")
(#xac1b023cba53b60d0c2382419fa66401 . "./code.cg2js/cg-scroll-bar.fasl")
(#x5990cdb6128868b28e643a83b8257f02 . "./code.cg2js/cg-web-browser.fasl")
(#xca9c983a566bd192210e5f8acc6cfbbc . "./code.cg2js/cg-dropping-outline.fasl")
(#x1a67ce944a793f046ae023f89e93bdde . "./code.cg2js/cg-font-dialog.fasl")
(#xa472acf2d6ec0cc00754e38e93601c55 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x2cf49e5de35a5616c2aa54ee8e058459 . "./code.cg2js/cg-yes-no-list.fasl")
(#x96f5d980d493c2e33f5d74ece208d6e7 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xf53f50ecdebbb5a9823c71571e7df3a2 . "./code.cg2js/cg-lamp.fasl")
(#xec684eae629b7dcf34c21d4707feda8c . "./code.cg2js/cg-drawable.fasl")
(#xa9d33276642e5b995ea7cce69b7d6efe . "./code.cg2js/cg-choice-list.fasl")
(#xefeb9621050155eb5d0d0f18dc525129 . "./code.cg2js/cg-curve.fasl")
(#xdbfceec867e3b7b03f447b19654df03e . "./code.cg2js/cg-base.fasl")
(#x0897017c754ea2fa3f4cfb2df816c395 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xa0152c8b8981f93dac66b3b0f6886be3 . "./code.cg2js/cg-printing.fasl")
(#x8fb8f8eaf3fe799302a1bcf9d5da0951 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x3b356078f08d6bd156594cc1cb8d432a . "./code.cg2js/cg-hotspot.fasl")
(#x82f670217febdab8f754b44d506587df . "./code.cg2js/cg-object-editor-layout.fasl")
(#x93272357243b676cc3c778a9f761dc87 . "./code.cg2js/cg-utility-dialog.fasl")
(#xae85026ae5d13c886d8709b1e3c94fd8 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x81bece9469973677449061498a0a9a9a . "./code.cg2js/cg-comtab.fasl")
(#x945b06171c09a768c68a6220207f9890 . "./code.cg2js/cg-os-widget.fasl")
(#x643420a08dc1b6f00bf8fbb2cea24cc6 . "./code.cg2js/cg-icon.fasl")
(#x7eebdcceda0c9b145fae6a7f5d9e70d5 . "./code.cg2js/cg-dev.fasl")
(#xe10913162ceac58bdc9ee871ae46d72f . "./code.cg2js/cg-calendar.fasl")
(#x76af8db04af64dbb3b97a7a40f98d073 . "./code.cg2js/cg-group-box.fasl")
(#x0ed59adc6b479fb5ccd950c3987b36dd . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x3aa3f6eac692a86a8e532f55f0f5be11 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xa8aa47f26630bf0f5dc65cc19f2df745 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x579c6df6e1b2e0ba5384513b964aea24 . "./code.cg2js/cg-nodes.fasl")
(#xdd188fc3a743fd6158e2c52806fe7041 . "./code.cg2js/cg-intersect.fasl")
(#x6f3498a15c44c3d70c2270f4586c0fa4 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x4dfb7f187279d74dc3909a43834a4d59 . "./code.cg2js/cg-class-support.fasl")
(#x479194be984a12ec130dc7008ec2aab9 . "./code.cg2js/cg-dialog-item.fasl")
(#xd2902c255d2a0e00a162b956456e4f39 . "./code.cg2js/cg-plot-widget.fasl")
(#x3119c99590fc9a69811495cebf624a3a . "./code.cg2js/cg-pixmap-printing.fasl")
(#xd7d03056c8253f819384c5bfb69fc5fa . "./code.cg2js/cg-up-down-control.fasl")
(#x86b7f338cf7ed3a9cb2bcf74c032eab7 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xd3d011a8648bb6f0f204c4c47eba4b0f . "./code.cg2js/cg-status-bar.fasl")
(#xb0cafe3f3c2925c03976ecf0f02774dc . "./code.cg2js/cg-class-grid.fasl")
(#xeb2e29af99431f9a8c1125e637491a8f . "./code.cg2js/cg-wrap-string.fasl")
(#x75d3ff1d9cf69c420013b25c3b94fdf1 . "./code.cg2js/cg-layout.fasl")
(#xc26a8416f84d2bfc0a5b5307c050e221 . "./code.cg2js/cg-lisp-widget.fasl")
(#x4cdea768c31239b3a5beb56f48d00f92 . "./code.cg2js/cg-button.fasl")
(#xc9e9a881b71d7d97358798e07e101b5c . "./code.cg2js/cg-string-dialog.fasl")
(#xc86ce3858bbcdb50f4c5f8b3f646d05c . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xa7b024e824184bd3f9c6eaf77d806320 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xbed456319245c4780abf4d19336f0ec3 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x616251aed6195bd3ec28c0958432a6ae . "./code.cg2js/cg-static-text.fasl")
(#x9f7cacf32e34c0906872fdb07188db17 . "./code.cg2js/cg-outline.fasl")
(#x96f9eef1768543b7e4a8803117252963 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x2d755457890121546cbe9395431a8ccf . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xb02218a5c1cfc31b098d6ff05d66d08d . "./code.cg2js/cg-picture-widget.fasl")
(#xdfdbd80b544012629b70d2dbe098056e . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x8d95bc23ca724ac7d12eee0e7dbf6b1d . "./code.cg2js/cg-chart-widget.fasl")
(#x7091c47f2bcd0932496f224b542d2d9b . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x4dec95d923cdf113f1f0febe7bc80034 . "./code.cg2js/cg-lisp-text.fasl")
(#x31e80d1137e13d37fd651df0d92ec1df . "./code.cg2js/cg-palette.fasl")
(#x40583bea303f4761e5891e9f61674591 . "./code.cg2js/cg-property.fasl")
(#x541982e87aa56e80757ae2ffa285295e . "./code.cg2js/cg-intersect-posbox.fasl")
(#x3de0270f31f5bbd7f95076a3b83aff19 . "./code.cg2js/cg-file-dialog.fasl")
(#x5cd39534edcc00f4e8d18f431b4bb260 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x4f45fa7832147887bd0bb2f8591cdec7 . "./code.cg2js/cg-get-pixmap.fasl")
(#xecaf4dc0bcd020401b1ddf2361ab4071 . "./code.cg2js/cg-tray.fasl")
(#x9b53ef979989379f2d12fbd9841e6351 . "./code.cg2js/cg-acache.fasl")
(#xa8ec9b01595f9d89bbae6db40923c062 . "./code.cg2js/cg-message-dialog.fasl")
(#xc60fad4697db540ecaf08fa2e35d846d . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xaf69b00b46b27b796991169d543d2759 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x7f9888c592dcfc85046344c7f0942764 . "./code.cg2js/cg-tab-control.fasl")
(#xd2c5bde38bc149082b7827258275bc38 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x880da5e506072ebb1a9fcb4f23dd2951 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x8df9249130da05e2056d590f04c4914d . "./code.cg2js/cg-find-string-dialog.fasl")
(#x4272f3e5195f925b8a8dd27694102e4b . "./code.cg2js/cg-menu.fasl")
(#x5844aa8f2c2bb9aaa6ea8caee36b6a55 . "./code.cg2js/cg-get-position.fasl")
(#xc913874a66e6965ae8d7c2a43a18cde8 . "./code.cg2js/cg-trackbar.fasl")
(#xef5d609a052def8e1ff3f2fc192b7616 . "./code.cg2js/cg-selected-object.fasl")
(#x9d01345c05e3c708111dd2708cef5ff3 . "./code.cg2js/cg-list-view.fasl")
(#xdc2642296edbd667bae944fa5a592c31 . "./code.cg2js/cg-html-widget.fasl")
(#x036768fb3d4629eb86831f19f84f6bf0 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x89212025b88804ef6699536a6e64e8e4 . "./code.cg2js/cg-text-widget.fasl")
(#xba4ac048011f5f7ef6a2b2b7ebe96ad5 . "./code.idejs/ide-base.fasl")
(#x06e45189fb9111d3d77980076ed127d1 . "./code.idejs/ide-process-browser.fasl")
(#x32d9e0f58fd5942b7dd3e83b4f587845 . "./code.idejs/ide-patch-dialog.fasl")
(#xf2e2aedc311ec064330e6120d7ea9bff . "./code.idejs/ide-keyshort.fasl")
(#xb4d4fd5d6584c0abdb6a20d77a719dd5 . "./code.idejs/ide-window-list.fasl")
(#x83ac428dac0a3663e9d1f56d540892f7 . "./code.idejs/ide.fasl")
(#xeb4fb9c55ed8c4cb6d59d69308159e66 . "./code.idejs/ide-editor.fasl")
(#x46ee387471da543e55e9b6c23ec1aab0 . "./code.idejs/ide-clipboard.fasl")
(#x1df6662dc4607e9e77f7769d08b35fc6 . "./code.idejs/ide-trace.fasl")
(#x1bc460d3248eaea1672842a8d70c0ca2 . "./code.idejs/ide-class-browser.fasl")
(#xe782365077d1bc22fe049367df17af22 . "./code.idejs/ide-grapher.fasl")
(#x98d4c43267027761ce73aa940cc2ec23 . "./code.idejs/ide-apropos.fasl")
(#xf445ab061b115ef20a960667ffb45883 . "./code.idejs/ide-navigator.fasl")
(#x931ad6872a2cdfb135784e8c16d5c461 . "./code.idejs/ide-find-definitions.fasl")
(#x84c39b4e0e974f679f6e6947cc389d0e . "./code.idejs/ide-project.fasl")
(#x1735e973100fbfdca306bdddca23576c . "./code.idejs/ide-class-grapher.fasl")
(#xe0c8b2713d4612a00eab9f61914bf640 . "./code.idejs/ide-acache.fasl")
(#xa4e00f1b89925f14949d4c46d990da7c . "./code.idejs/ide-find-in-files.fasl")
(#x0ce85e778b5d0c2f40d35dd668adee50 . "./code.idejs/ide-inspector.fasl")
(#xc4996b962c6685085b19e10886b37b06 . "./code.idejs/ide-intro.fasl")
(#x8d8c3309279128c1f81afac5cd53dd53 . "./code.idejs/ide-package-list.fasl")
(#xeb849c867afe526409140f8ac1daa379 . "./code.idejs/ide-dev.fasl")
(#xbf6dcba8038de5f70f59776b6bdedc84 . "./code.idejs/ide-form.fasl")
(#xf3f59eb0bf604c0b34476c9897472f9c . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
